/* lib/install.p
 *
 * tailor a ProTop version 3 install
 *
 */

define new global shared variable dbgMode   as integer no-undo.
define new global shared variable waitLimit as decimal no-undo.

define variable envDLC     as character no-undo format "x(250)" view-as fill-in size 40 by 1 label "    OpenEdge".
define variable envPT3     as character no-undo format "x(250)" view-as fill-in size 40 by 1 label "    ProTop 3".

define variable envLOG     as character no-undo format "x(250)" view-as fill-in size 40 by 1 label "   Log Files".
define variable envTMP     as character no-undo format "x(250)" view-as fill-in size 40 by 1 label "  Temp Files".
define variable envRPT     as character no-undo format "x(250)" view-as fill-in size 40 by 1 label "     Reports".

define variable envPROXY   as character no-undo.
define variable envHOST    as character no-undo.
define variable envUPD     as character no-undo.

define variable envPRP     as character no-undo.		/* PROPATH for .pl files	*/

define variable assemblyPath as character no-undo.

define variable srvName    as character no-undo.
define variable pvers      as character no-undo.

define variable custId     as character no-undo format "x(250)" view-as fill-in size 15 by 1 label " Customer ID".

define new global shared
       variable portalProxy   as character no-undo format "x(250)" view-as fill-in size 40 by 1 label "       Proxy".

define variable PT3env     as character no-undo.		/* bin/protopenv -- not $PROTOP		*/

define variable DS         as character no-undo.
define variable ok         as logical   no-undo.
define variable useSockets as logical   no-undo.

define variable bkupName   as character no-undo.
define variable stubDir    as character no-undo.
define variable stubName   as character no-undo.
define variable baseName   as character no-undo.
define variable saveName   as character no-undo.
define variable scriptName as character no-undo.

define stream instStream.
define stream inStrm.


/* allow the debug mode to be set externally
 */

define variable x as integer no-undo.

x = integer( os-getenv( "DBGMODE" )) no-error.

if x <> ? and x > 0 and x < 9 then
  dbgMode = x.
 else
  dbgMode = 3.

{lib/protoplib.i}

run lib/protoplib.p persistent.							/* load protop infrastructure library           		*/

publish "setLogName" ( "pt3inst.log" ).

logMsg( 0, substitute( "ProTop Installer {lib/ptversion.i}, DBGMODE=&1", dbgMode )).

{lib/osinfo.i}

if os-getenv( "USESOCKETS" ) = "no" then
  useSockets = no.
 else
  useSockets = yes.

run ssg/sausage02.p persistent.
run ssg/sausage06.p persistent.


/* main body
 */

pause 0 before-hide.

file-info:file-name = ".".						/* we might need to try "protop.p" if Windows installs get cranky	*/

if file-info:full-pathname <> ? then
  envPT3 = file-info:full-pathname.
 else
  do:
    logMsg( 0, "Unknown install path, shamefully quitting." ).
    if session:batch = no then message "Unknown install path, shamefully quitting.".
    quit.
  end.

DS = ( if opsys begins "win" then "~\" else "/" ).

if lookup( ".", propath ) <= 0 then					/* everything is relative to the current directory			*/
  propath = propath + ",.".

/* in order to use System.Management.dll Progress needs to know where to look for it.
 */

if opsys begins "win" and integer( entry( 1, proversion, "." )) >= 11 then
  assemblyPath = "-assemblies bin".					/* #PTASMBLY# in estub/protop.pf					*/

if session:batch = no then 
  do:
    run lib/chkscreen.p.
    message "ProTop 3 is being installed in:" envPT3.
    message "Detailed log is: pt3inst.log".
  end.

logMsg( 0, substitute( "ProTop 3 is being installed in: &1, DS = &2", envPT3, DS )).

assign
  envDLC   = os-getenv( "DLC" )
  envPROXY = os-getenv( "PROXY" )
  envHOST  = os-getenv( "PTHOST" )
  envUPD   = os-getenv( "PTUPD" )
.

/* no! make all of these relative to $PROTOP
 *
 *  envLOG   = os-getenv( "LOGDIR" )
 *  envTMP   = os-getenv( "TMPDIR" )
 *  envRPT   = os-getenv( "RPTDIR" )
 */

if envDLC = ? then
  do:
    logMsg( 0, "Unknown $DLC, shamefully quitting." ).
    if session:batch = no then message "Unknown $DLC, shamefully quitting.".
    quit.
  end.

if envPROXY = ? then envPROXY = "".
portalProxy = envPROXY.

if envHOST  = ? or envHOST = "" then envHOST  = "demo.wss.com".
if envUPD   = ? or envUPD  = "" then envUPD   = "demo.wss.com".

logMsg( 0, substitute( "Values provided: [DLC=&1] [PROXY=&2] [PORTAL=&3] [UPDATES=&4]", envDLC, envPROXY, envHOST, envUPD )).

/* get some basics about the os that we are insalling on
 */

run osInfo.

find first tt_osInfo no-error.
if available tt_osInfo then
  logMsg( 0, substitute( "OSInfo: [host=&1] [os=&3] [osVersion=&3] [osKernel=&4] [cpu=&5] [clock=&6] [numcpu=&7] [RAM=&8]", tt_osInfo.hostName, tt_osInfo.osName, tt_osInfo.osVersion, tt_osInfo.keVersion, tt_osInfo.modelCPU, tt_osInfo.clkCPU, tt_osInfo.numCPU, tt_osInfo.memTotal )).
 else
  logMsg( 0, substitute( "OSInfo: <not available>" )).

file-info:file-name = substitute( "&1&2version", envDLC, DS ).
if file-info:full-pathname = ? then
  pvers = proversion.
 else
  do:
    input stream inStrm from value( file-info:full-pathname ).
    do on error undo, leave
       on endkey undo, leave:
      import stream inStrm ^ ^ pvers.
    end.
    input stream inStrm close.
  end.

logMsg( 0, substitute( "proInfo: [proversion=&1] [bits=&2]", pvers, max( "32", "{&PROCESS-ARCHITECTURE}" ))).

/* set some sensible initial values relative to $PROTOP
 */

assign
  envLOG = envPT3 + DS + "log"
  envTMP = envPT3 + DS + "tmp"
  envRPT = envPT3 + DS + "rpt"
.

/* the actual work of installing protop
 */

envDLC = right-trim( envDLC, DS ).
envPT3 = right-trim( envPT3, DS ).
envLOG = right-trim( envLOG, DS ).
envTMP = right-trim( envTMP, DS ).
envRPT = right-trim( envRPT, DS ).

/* figure out which .pl file to use
 */

/* old method defeated by too many permutations of oe10 rcode :(
 *
 *   note the lack of "," compared to current style - estub\protop.ini needs
 *   to reflect whichever approach is used
 *
 * if entry( 1, proversion, "." ) = "10" and is64bits() = true then
 *   envPRP = substitute( "protop-v&1-64.pl", entry( 1, proversion, "." )).		/* oe10 64 bit r-code */
 *  else
 *   envPRP = substitute( "protop-v&1.pl", entry( 1, proversion, "." )).		/* oe11, oe12 etc use PROLIB		*/
 */

/* Unix:
 *    PROPATH=#ENVPRP#${PROTOP}
 *
 * Windows:
 *    PROPATH=#ENVPRP##ENVPT3#,#ENVDLC#,#ENVDLC#\bin,#ENVDLC#\gui
 */

if entry( 1, proversion, "." ) = "10" then
  envPRP = "".										/* oe10 NO PROLIB, use x-code	*/
 else
  envPRP = substitute( "protop-v&1.pl", entry( 1, proversion, "." )).			/* oe11, oe12 etc use PROLIB		*/

if envPRP <> "" then
  do:
    if opsys = "unix" then
      envPRP = envPRP + ":".
     else
      envPRP = envPRP + ",".
  end.

logMsg( 0, substitute( "Initial working values: [DLC=&1] [LOGDIR=&2] [TMPDIR=&3] [RPTDIR=&4] [CUSTID=&5] [ENVPT3=&6] [ENVPRP=&7]", envDLC, envLOG, envTMP, envRPT, custId, envPT3, envPRP )).

os-create-dir
  value( envPT3 + "/bin" )
  value( envPT3 + "/dbanalys" )
  value( envPT3 + "/etc" )
  value( envPT3 + "/ptdefs" )
  value( envLOG )
  value( envTMP )
  value( envRPT )
.

/*
 * if os-error <> 0 then
 *   do:
 *     message
 *       "Error:" os-error "occurred when creating required directories:" skip
 *       envPT3 + "/bin" envLOG envTMP envRPT skip
 *       skip(1)
 *       "Canceling installation" skip
 *       view-as alert-box
 *     .
 *     quit.
 *   end.
 */

logMsg( 0, substitute( "Created &1, &2, &3, &4, &5, &6, and &7",
  envPT3 + "/bin",
  envPT3 + "/dbanalys",
  envPT3 + "/etc",
  envPT3 + "/ptdefs",
  envLOG,
  envTMP,
  envRPT
  )
).


/* prompt the user for information or overrides if necessary, sub-directories must exist first
 *
 */

if custID = ? or custId = "" then
  do:
    file-info:file-name = envPT3 + DS + "etc" + DS + "custid.cfg".
    if file-info:full-pathname <> ? then
      do on error undo, leave
         on endkey undo, leave:
        input stream inStrm from value( file-info:full-pathname ).
        import stream inStrm unformatted custId.
        input stream inStrm close.
      end.
    logMsg( 0, substitute( "Found an existing custId:", custId )).
  end.

if custId = ? then custId = "".

run lib/inschui.p (
  input-output custId,		/* picked up from etc/dblist.cfg if it exists, otherwise blank	*/
  input-output envHOST		/* from os-getenv( "PTHOST" )					*/
).

if custId = ? then
  custId = "".
 else
  custId = trim( custId ).

output stream instStream to value( envPT3 + DS + "etc" + DS + "custid.cfg" ).
put stream instStream unformatted custId skip.
output stream instStream close.


/* get ready to tailor
 */

if opsys = "unix" then
  assign
    PT3env  = envPT3 + "/bin/protopenv"
    stubDir = envPT3 + "/ustub"
  .
 else
  assign
    PT3env  = envPT3 + "~\bin~\protopenv.bat"
    stubDir = envPT3 + "~\wstub"
  .

logMsg( 0, substitute( "Tailoring working values: [protopenv=&1] [stubdir=&2]", pt3env, stubdir )).

bkupName = substitute( "tmp" + DS + "etc.&1.&2.tar", string( month( today ), "99" ), string( day( today ), "99" )).
file-info:file-name = bkupName.
if file-info:full-pathname = ? then
  do:
    logMsg( 0, substitute( "Backing up etc to &1", bkupName )).
    os-command silent value( "tar cvf " + bkupName + " etc > tmp" + DS + "bkup.err 2>&1" ).
  end.

if session:batch = no then message "Tailoring...".

logMsg( 0, substitute( "Copying stub files from: &1", stubDir )).

input stream instStream from os-dir( stubDir ).

binstub_loop: repeat:

  import stream instStream ^ stubName.

  if substring( stubName, length( stubName ), 1 ) = "." then		/* if it ends in "." it is either "." or ".." or	*/
    next binstub_loop.							/* crazily named and unworthy of our attention		*/

  logMsg( 0, substitute( "Tailoring: &1", stubName )).

  /* special cases
   */

  if opsys = "unix" then
    do:

      scriptName = replace( stubName, "ustub", "bin" ).
      if stubName matches "*uemacs*" then
        do:
          os-copy value( stubName ) value( "bin/uemacs" ).
          next binstub_loop.						/* don't try to tailor uemacs - it a binary		*/
        end.

    end.
   else
    do:

      if r-index( stubName, ".INI" ) = ( length( stubName ) - 3 ) then
        scriptName = replace( stubName, "wstub", "etc" ).
       else
        scriptName = replace( stubName, "wstub", "bin" ).

    end.

  run tailor( stubName, scriptName ).

  logMsg( 0, substitute( " Finished: &1", scriptName )).

end.		/* binstub_loop	*/

input stream instStream close.


/* copy files from estub to etc, tailor along the way
 *
 */

file-info:file-name = "estub".
if file-info:full-pathname = ? then
  do:
    logMsg( 0, "No estub directory was found" ).
  end.
 else
  do:

    logMsg( 0, "Tailoring estub" ).

    input stream instStream from os-dir( "estub" ).

    estub_loop: repeat:

      import stream instStream ^ stubName.

      /* special cases
       */

      if substring( stubName, length( stubName ), 1 ) = "." then	/* if it ends in "." it is either "." or ".." or	*/
        next estub_loop.						/* crazily named and unworthy of our attention		*/

      baseName = substring( stubName, r-index( stubName, "estub" ) + 6 ).

      /* skip these, do NOT copy or over-write
       */

      if lookup( baseName, "custid.cfg,dblist.cfg" ) > 0 then
        next estub_loop.

      /* do NOT tailor (binary files, encrypted files)
       */

      if lookup( baseName, "protop.ico,alert.cfg,highlight.cfg,healthcheck.cfg" ) > 0 then
        do:
          os-copy value( "estub/" + baseName ) value( "etc/" + baseName ).
          next estub_loop.
        end.

      /* os specific variants
       */

      if baseName = "schedule.cfg" then
        next estub_loop.

      if baseName = "schedule.cfg.unix" then
        do:
          if opsys = "unix" then
            do:
              logMsg( 0, substitute( "copying &1 to etc/schedule.cfg", baseName )).
              os-copy value( "estub/" + baseName ) value( "etc/schedule.cfg" ).
            end.
           else
            do:
              logMsg( 0, substitute( "skipping &1", baseName )).
            end.
          next estub_loop.
        end.

      if baseName = "schedule.cfg.windows" then
        do:
          if opsys begins "win" then
            do:
              logMsg( 0, substitute( "copying &1 to etc/schedule.cfg", baseName )).
              os-copy value( "estub/" + baseName ) value( "etc/schedule.cfg" ).
            end.
           else
            do:
              logMsg( 0, substitute( "skipping &1", baseName )).
            end.
          next estub_loop.
        end.

      /* if there is an existing file in the LOOKUP() list make a backup before overwriting it
       */

      file-info:file-name = "etc/" + baseName.
      if file-info:full-pathname <> ? then
        do:

          if lookup( baseName, "protop.pf" ) > 0 then
            do:
              saveName = file-info:full-pathname + substitute( ".&1.&2", string( month( today ), "99" ), string( day( today ), "99" )).
              logMsg( 0, substitute( "&1 already exists in etc, saving a copy as &2", baseName, saveName )).
              file-info:file-name = saveName.
              if file-info:full-pathname = ? then
              os-copy value( "etc/" + baseName ) value( saveName ).
              /* proceed with tailoring		*/
            end.

        end.

      /* standard tailoring
       */

      logMsg( 0, substitute( "Tailoring: &1", stubName )).
      scriptName = replace( stubName, "estub", "etc" ).
      run tailor( stubName, scriptName ).

    end.	/* estub_loop	*/

    input stream instStream close.

  end.

/* end. */

if opsys = "unix" then
  do:
    logMsg( 0, "Fixing permissions" ).
    os-command silent value( "chmod 755 " + envPT3 + "/bin/" + "*" ).
  end.
 else
  do:
    /* ... (PROVERSION BEGINS "10.2B" OR INTEGER(ENTRY (1, PROVERSION, ".")) GE 11) ... */
    if integer( entry( 1, proversion, "." )) >= 11 then
      do:
        logMsg( 0, substitute( "Updating registry with ProTop install directory: &1", envPT3 )).
        run lib/setpt3dir.p ( envPT3 ).			/* set registry key to locate ProTop install directory -- contributed by Mike @ Consultingwerk	*/
      end.
  end.


file-info:file-name = envTMP + "/dbmonitor.flg".

if file-info:full-pathname = ? then
  do:

    if opsys = "unix" then
      do:
        logMsg( 0, "Starting dbmonitor" ).
        os-command silent value( "bin/dbmonitor.sh" ). 
      end.
     else
      do:

        logMsg( 0, "Installing dbmonitor" ).

        /* nssm_setup needs to run as admin -- right now I don't have a good way to do that
         *
         * 	os-command value( "runas /user:administrator bin~\nssm_setup.bat" ).
         *
         * is close but it prompts for a password even if you are already admin
         *
         * so, for now, I'm going to assume most people are installing as admin
         * and just go ahead and run it
         */

	os-command silent value( "bin~\nssm_setup.bat" ).

      end.

  end.
 else
  do:

    logMsg( 0, "Restarting monitors" ).

    if opsys = "unix" then
      os-command silent value( "rm -f "  + envTMP +  "/*.flg" ).
     else
      os-command silent value( "del /q " + envTMP + "~\*.flg 2>NUL" ).

  end.


logMsg( 0, "Finished!" ).
logMsg( 0, "" ).

if session:batch then
  return.
 else
  quit.


define stream stubStream.
define stream scriptStream.

procedure tailor:

  define input parameter srcFile as character no-undo.
  define input parameter dstFile as character no-undo.

  define variable scriptBody as character no-undo.

  if session:batch = no and dbgMode > 3 then message "Tailoring:" srcFile "==>" dstFile.

  /* copy-lob would be nice but we are still supporting v9...
   */

  input stream stubStream from  value( srcFile ).
  output stream scriptStream to value( dstFile ).

  repeat:

    scriptBody = "".
    import stream stubStream unformatted scriptBody.

    scriptBody = replace( scriptBody, "#PT3ENV#", PT3env ).
    scriptBody = replace( scriptBody, "#ENVDLC#", envDLC ).
    scriptBody = replace( scriptBody, "#ENVPRP#", envPRP ).
    scriptBody = replace( scriptBody, "#ENVPT3#", envPT3 ).
    scriptBody = replace( scriptBody, "#ENVLOG#", envLOG ).
    scriptBody = replace( scriptBody, "#ENVTMP#", envTMP ).
    scriptBody = replace( scriptBody, "#ENVRPT#", envRPT ).
    scriptBody = replace( scriptBody, "#PTHOST#", envHOST ).
    scriptBody = replace( scriptBody, "#PTUPD#",  envUPD ).

    scriptBody = replace( scriptBody, "#PTASMBLY#", assemblyPath ).

    if index( stubName, "protop.cfg" ) > 0 and portalProxy <> "" then
      scriptBody = replace( scriptBody, "#PROXY#",  "proxy	" + portalProxy ).
     else
      scriptBody = replace( scriptBody, "#PROXY#",  portalProxy ).

    put stream scriptStream unformatted scriptBody skip.
    if scriptBody = "" then put stream scriptStream unformatted skip(1).

  end.
  input stream stubStream close.
  output stream scriptStream close.

  return.

end.


/* move this to lib/protoplib.p someday
 */

function is64bits returns logical ():				/* thanks to Jeff Ledbetter of Roundtable Software for this gem!	*/

  define variable i1 as int64 no-undo.
  define variable i2 as int64 no-undo.

  define variable m  as memptr no-undo.

  i1 = 0x100000000.						/* Hex value of largest 32-bit unsigned int + 1 */

  set-pointer-value( m ) = i1.

  i2 = get-pointer-value( m ).

  return ( i1 = i2 ).

end.
