#!/bin/sh
#
# sendalert.sh s2k -type info -m testMetric -msg "testing, testing, 1, 2, 3..."
#

. #ENVPT3#/bin/protopenv

TMPDIR=${TMPDIR-/tmp}
LOGDIR=${LOGDIR-/tmp}

#UN-comment for DEBUG
# ECHO=echo

cd ${PROTOP}

if [ ! -d ${TMPDIR} ]
then
        mkdir ${TMPDIR}
fi

if [ ! -d ${LOGDIR} ]
then
	mkdir ${LOGDIR}
fi

errmsg()
{
	echo ""
	echo "${1}"
	echo ""
}

###### PARAM SYNTAX: -param "friendlyName|alertType|alertMetric|alertMessage"
###### alertType = info,alert,alarm,page
###### alertMetric and alertMessage are arbitrary text

USAGE="$0 <friendlyName> [-type <alertType=alert>] [-m <alertMetric=sendAlert>] [-msg <\"Alert msg in quotes\">]"
TYPE=alert
METRIC=sendAlert
MSG="No message text"

# P1 is a "friendly name" in etc/dblist.cfg
P1=${1}
shift

if [ -z "${P1}" ]
then
	errmsg "Please specify a friendly name from etc/dblist.cfg"
	exit 1
fi

# Help
if [[ ${P1} = "-h" ]]; then echo $USAGE;exit 0;fi

while [[ "$@" != "" ]]
do
  case $1 in
    -type) shift;TYPE="${1}";shift;;
    -m   ) shift;METRIC="${1}";shift;;
    -msg ) shift;MSG="${1}";shift;;
    *    ) echo "Unknown option $1";echo $USAGE;exit 1 ;;
  esac
done

# Check etc/dblist.cfg for a matching "friendly name"
#

if [ -f ${PROTOP}/etc/dblist.cfg ]			# search using "pipe" as the delimiter
then
	XDB=$(grep "^${P1}|" ${PROTOP}/etc/dblist.cfg | awk -F"|" '{print $2}' 2>/dev/null)
fi

# if XDB has a value then we found a friendly name and the path is in XDB #

if [ -z "${XDB}" ]					# no friendly name?
then
	errmsg "Friendly name ${P1} does not exist."
	exit 1
else
	PARAM="${P1}|$TYPE|$METRIC|$MSG"	# pass the friendly name into protop.p
fi

$ECHO ${DLC}/bin/_progres -b -p util/sendalert.p ${PFLIST} -param "${PARAM}" -T ${TMPDIR} >> $LOGDIR/sendalert.log 2>&1
